
/*---------------------------------------------------------------------------

    Filename: date_range.js
    Project:  Data Analysis Extensions
    (C) Copyright 2008, Kirix Corporation, All Rights Reserved.

    Summary:        Engine for Date Range Analysis extension

    Developer(s):   Aaron L. Williams

---------------------------------------------------------------------------*/


class StatsAccumulator
{
    // class: StatsAccumulator
    // 
    // summary: this class analyzes a table and accumulates 
    // date statistics on the table

    function StatsAccumulator()
    {
        // function: StatsAccumulator()
        //
        // summary: constructor function that simply initializes 
        // the items variable

        m_items = new Array();
    }

    function analyzeTable(database, source, fields)
    {
        // function: analyzeTable(database : String, 
        //                        source : String, 
        //                        fields : Array)
        // 
        // summary: this function takes a database connection, source 
        // table in the database, and a list of fields in that table, 
        // and accumulates statistics on that table

        // if we don't have a valid database, return
        if (!database)
            return;

        // if we don't have a source table, return   
        if (!source)
            return;
            
        // if we don't have any fields, return
        if (!fields || fields.length == 0)
            return;
        
        // store the source path
        m_source_table = source;
        
        // store the source fields
        m_source_fields = fields;

        // create a new job to add to the job queue
        m_job = HostApp.createJob();
        m_job.setTitle("Date Range Analysis (" + m_source_table + ")");
        m_job.start();
        
        // construct the SQL select statement
        var sql = "";
        sql += "SELECT ";
        sql += fields.toString() + " ";
        sql += "FROM ";
        sql += source;
 
        // note: perform a simple select query on the fields; we'll 
        // accumulate the statistics when we iterate through the results; 
        // while this is less effecient than having the query do the
        // work, it enables us to pull down and analyze multiple fields 
        // at the same time more easily than with SQL
        var result = database.execute(sql);
 
        // while we have result rows, iterate through them
        while (result.next())
        {
            // for each row, get the values for each of
            // the fields we're tracking
            for (var i in fields)
            {
                // get the year and month for the field
                var value = new Date(result.getValue(fields[i])); 
                var year = value.getFullYear();
                var month = value.getMonth() + 1;
                
                // add/increment the statist for the year and
                // month in question
                incrementItemStat(fields[i], year, month);
                
                // see if the job is cancelled; if it is, return
                if (m_job.isCancelling())
                {
                    m_job.cancel();
                    break;
                }
            }
        }    
    }

    function saveOutputTable(database, output)
    {
        // function saveOutputTable(database : String,
        //                          output : String)
        //
        // summary: this function writes the accumulated statistics
        // to the specified output table

        // if we don't have a valid database, return
        if (!database)
            return;
    
        // if we don't have a specified output, return
        if (!output)
            return;

        // create the SQL for creating the output table       
        var sql = "";

        sql +=
        "
            CREATE TABLE " + output + " 
            (
                STAT_YEAR       NUMERIC(4,0),
                STAT_MONTH      NUMERIC(4,0),
        ";
 
        for (var i in m_source_fields)
        {
            sql += m_source_fields[i] + " NUMERIC(12,0),";
        }

        sql +=
        "
                TOTAL           NUMERIC(12,0)
            );
        ";
 
        // create the output table
        database.execute(sql);
        
        // create a bulk inserter to insert the results
        // into the output table
        var first_item = getFirstItemByDate();
        var last_item = getLastItemByDate();
        
        var year = first_item.m_year;
        var month = first_item.m_month;
        var output_fields = new Array();

        output_fields.push("STAT_YEAR");
        output_fields.push("STAT_MONTH");

        for (var i in m_source_fields)
            output_fields.push(m_source_fields[i]);

        output_fields.push("TOTAL");
        
        var inserter = database.bulkInsert(output, output_fields.toString());
        
        // iterate through the statistics and add them to the output
        while (1)
        {
            var total_count = 0;

            inserter["STAT_YEAR"] = year;
            inserter["STAT_MONTH"] = month;
        
            for (var f in m_source_fields)
            {
                var item = getItemStat(m_source_fields[f], year, month);
                total_count += item.m_count;
 
                inserter[item.m_field] = item.m_count;
            }

            inserter["TOTAL"] = total_count;
            inserter.insertRow();
        
            if (year == last_item.m_year && month == last_item.m_month)
                break;

            // reset the count and increment the month/year
            total_count = 0;
            
            month++;
            if (month > 12)
            {
                month = 1;
                year++;
            }
            
            if (m_job && m_job.isCancelling())
            {
                // cancel the job
                m_job.cancel();
                break;
            }
        }

        // finish inserting the results into the output table
        inserter.finishInsert();

        if (m_job)
            m_job.finish();
    }

    function incrementItemStat(field, year, month)
    {
        // function incrementItemStat(field : String, 
        //                            year : Number, 
        //                            month : Number)
        //
        // summary: increments the count for a given field, year, 
        // and month; if the item doesn't already exist, the function 
        // creates a new item

        // get the key for the field, year, and month
        var key = getKey(field, year, month);

        // if we don't have an item tracking the statistic,
        // create one
        if (!m_items[key])
        {
            var ds = new StatsAccumulatorItem;
            ds.m_field = field;
            ds.m_year = year;
            ds.m_month = month;
            ds.m_count = 0;

            m_items[key] = ds;
        }

        // increment the count
        m_items[key].m_count++;
    }

    function getItemStat(field, year, month)
    {
        // function getItemStat(field : String, 
        //                      year : Number, 
        //                      month : Number) : StatsAccumulatorItem
        //
        // summary: returns the statistics for a given field, year, 
        // and month
    
        // create a key
        var key = getKey(field, year, month);
        
        // if we hae an item stat corresponding to
        // the key, return it
        if (m_items[key])
            return m_items[key];

        // otherwise, create an item statistic corresponding
        // to the key, set the count to zero, and return it
        var ds = new StatsAccumulatorItem;
        ds.m_field = field;
        ds.m_year = year;
        ds.m_month = month;
        ds.m_count = 0;
        
        return ds;
    }
    
    function getFirstItemByDate()
    {
        // function getFirstItemByDate() : StatsAccumulatorItem
        // 
        // summary: returns the first StatsAccumulatorItem by date 
        // from the list of accumulated statistics
    
        // start with a large year and month
        var item = new StatsAccumulatorItem;
        var min_year = 1000000;
        var min_month = 12;

        // iterate through the statistic items
        for (var i in m_items)
        {
            // get the year and month for the current item
            var cur_year = m_items[i].m_year;
            var cur_month = m_items[i].m_month;

            // if the year isn't less than the minimum year up
            // to this point, move on; if the year is the same
            // as the minimum year up to this point, but the
            // month isn't less, move on
            if ((cur_year > min_year) || 
                (cur_year == min_year && cur_month > min_month))
            {
                continue;
            }

            // set the new minimum year, month, and item
            min_year = cur_year;
            min_month = cur_month;
            item = m_items[i];
        }
        
        // return the minimum item
        return item;
    }
    
    function getLastItemByDate()
    {
        // function getFirstItemByDate() : StatsAccumulatorItem
        //
        // summary: returns the first StatsAccumulatorItem item 
        // by date from the list of accumulated statistics    
    
        // start with a small year and month
        var item = new StatsAccumulatorItem;
        var max_year = -1000000;
        var max_month = 1;

        // iterate through the statistic items        
        for (var i in m_items)
        {
            // get the year and month for the current item        
            var cur_year = m_items[i].m_year;
            var cur_month = m_items[i].m_month;

            // if the year isn't more than the maximum year up
            // to this point, move on; if the year is the same
            // as the maximum year up to this point, but the
            // month isn't more, move on
            if ((cur_year < max_year) || 
                (cur_year == max_year && cur_month < max_month))
            {
                continue;
            }

            // set the new maximum year, month, and item
            max_year = cur_year;
            max_month = cur_month;
            item = m_items[i];
        }
        
        // return the maximum item
        return item;
    }
    
    function getKey(field, year, month)
    {
        // function getKey(field : String, 
        //                 year : Number, 
        //                 month : Number)
        // 
        // summary: returns a string from the input field, year, 
        // and month; used to construct a has string for accumulating 
        // statistics

        var key = "F:" + field + "Y:" + year + "M:" + month;
        return key;
    }

    // member variables
    var m_job;
    var m_items;
    var m_source_table;
    var m_source_fields;
}


class StatsAccumulatorItem
{    
    // class: StatsAccumulatorItem
    //
    // summary: this class stores a single item; used by
    // the StatsAccumulator class to store statistics

    function getText()
    {
        // function getText()
        // 
        // summary: returns the class data as a string
    
        var s = "";
        s += "FIELD: " + m_field + "\n";
        s += "YEAR: " + m_year + "\n";
        s += "MONTH: " + m_month + "\n";
        s += "COUNT: " + m_count + "\n";
        
        return s;
    }
    
    // member variables
    var m_field;
    var m_year;
    var m_month;
    var m_count;    
}

