
/*---------------------------------------------------------------------------

    Filename: date_range_form.js
    Project:  Data Analysis Extensions
    (C) Copyright 2008, Kirix Corporation, All Rights Reserved.

    Summary:        Form for Date Range Analysis extension

    Developer(s):   David Z. Williams

---------------------------------------------------------------------------*/


include "fieldpicker.js";
include "date_range.js";


class DateRangeForm extends Form
{
    // class: DateRangeForm
    //
    // summary: top-level form used for specifying the input for the
    // date range analysis

    function DateRangeForm()
    {
        // function: DateRangeForm()
        //
        // summary: constructor function that defines the form    

        // call the constructor on the base Form object and
        // set the minimum size
        super("Date Range Analysis", 0, 0, 500, 420);
        this.setMinSize(500, 420);

        // create helper text
        var helper_label = new Label("Please select the table and date fields that you would like to summarize.");
        
        // create source textbox
        m_source_textbox = new TextBox;
        m_source_textbox.textChanged.connect(this, onSourceTextChanged);
        
        // create source browse button
        var source_browse_button = new Button("Browse...");
        source_browse_button.click.connect(this, onSourceBrowseClicked);
        
        // create field picker
        m_field_picker = new FieldPicker;
        
        // create output textbox
        m_output_textbox = new TextBox;
        
        // create output browse button
        var output_browse_button = new Button("Browse...");
        output_browse_button.click.connect(this, onOutputBrowseClicked);
        
        // create run button
        var run_button = new Button("Run");
        run_button.click.connect(this, onRunClicked);
        
        // create cancel button
        var cancel_button = new Button("Cancel");
        cancel_button.click.connect(this, onCancelClicked);
        
        // create top layout
        var source_layout = new BoxLayout(Layout.Horizontal);
        source_layout.add(new Label("Source Table:"), 0, Layout.Center);
        source_layout.addSpacer(5);
        source_layout.add(m_source_textbox, 1, Layout.Center);
        source_layout.addSpacer(8);
        source_layout.add(source_browse_button);
        
        // create output layout
        var output_layout = new BoxLayout(Layout.Horizontal);
        output_layout.add(new Label("Output Table:", 0, 0, 75), 0, Layout.Center);
        output_layout.addSpacer(5);
        output_layout.add(m_output_textbox, 1, Layout.Center);
        output_layout.addSpacer(8);
        output_layout.add(output_browse_button);

        // create button layout
        var button_layout = new BoxLayout(Layout.Horizontal);
        button_layout.addStretchSpacer();
        button_layout.add(run_button);
        button_layout.addSpacer(8);
        button_layout.add(cancel_button);
        
        // create main layout
        var main_layout = new BoxLayout(Layout.Vertical);
        main_layout.addSpacer(8);
        main_layout.add(helper_label, 0, Layout.Expand | Layout.Left | Layout.Right, 8);
        main_layout.addSpacer(4);
        main_layout.add(new Line(Line.Horizontal, 0, 0, 1, 1), 0, Layout.Expand | Layout.Left | Layout.Right, 8);
        main_layout.addSpacer(12);
        main_layout.add(source_layout, 0, Layout.Expand | Layout.Left | Layout.Right, 8);
        main_layout.addSpacer(8);
        main_layout.add(m_field_picker, 1, Layout.Expand | Layout.Left | Layout.Right, 8);
        main_layout.addSpacer(8);
        main_layout.add(output_layout, 0, Layout.Expand | Layout.Left | Layout.Right, 8);
        main_layout.addSpacer(8);
        main_layout.add(new Line(Line.Horizontal), 0, Layout.Expand);
        main_layout.addSpacer(8);
        main_layout.add(button_layout, 0, Layout.Expand | Layout.Left | Layout.Right, 8);
        main_layout.addSpacer(8);

        // set the main layout for the form        
        setLayout(main_layout);
    }
    
    function onSourceTextChanged(sender, args)
    {
        // function: onSourceTextChanged(sender : Object,
        //                               args : Array)
        //
        // summary: sets the source table for the field picker
    
        m_field_picker.setTable(args.text);
    }
    
    function onSourceBrowseClicked()
    {
        // function: onSourceBrowseClicked()
        //
        // summary: when the source browse button is clicked,
        // show a project file dialog
    
        var dlg = new ProjectFileDialog(ProjectFileDialog.Open);
        dlg.setCaption("Select Source Table");
        if (dlg.showDialog())
            m_source_textbox.setText(dlg.getPath());
    }
    
    function onOutputBrowseClicked()
    {
        // function: onOutputBrowseClicked()
        //
        // summary: when the source browse button is clicked,
        // show a project file dialog
 
        var dlg = new ProjectFileDialog(ProjectFileDialog.Open);
        dlg.setCaption("Select Output Table");
        if (dlg.showDialog())
            m_output_textbox.setText(dlg.getPath());
    }
    
    function onRunClicked()
    {
        // function: onRunClicked()
        //
        // summary: when the run button is clicked, hide the
        // form and start the date range analysis

        var output_path = m_output_textbox.getText();
        var source_path = m_field_picker.getTable();
        var source_fields = m_field_picker.getSelectedFields();
        
        // check to make sure the user has typed in an output table
        if (output_path.length == 0)
        {
            alert("Please enter a valid output table to continue.");
            return;
        }
        
        // check to make sure we're not overwriting an existing table
        var db = HostApp.getDatabase();
        if (db.exists(output_path))
        {
            var dlg_res = alert("The specified output table already exists.  Would you like to overwrite it?",
                                HostApp.getFrameCaption(),
                                DialogResult.YesNo);
                                
            if (dlg_res == DialogResult.No)
                return;
        }
        
        // hide the form
        this.show(false);
        
        // run the date range analysis
        var date_stats = new StatsAccumulator();
        date_stats.analyzeTable(db, source_path, source_fields);
        date_stats.saveOutputTable(db, output_path);
        
        // refresh the project panel and open the output table
        HostApp.refresh();
        HostApp.open(output_path);
        
        // close out
        close();
    }
    
    function onCancelClicked()
    {
        // function: onCancelClicked()
        //
        // summary: when the cancel button is clicked, either hide
        // the import form or exit the application, depending
        // on whether or not we are running as an extension
 
        // close out
        close();
    }
    
    function close()
    {
        // function: close()
        //
        // summary: close the application
 
        if (Extension.isContextPackage())
            this.show(false);
             else
            Application.exit();
    }

    // member variables
    var m_field_picker;
    var m_source_textbox;
    var m_output_textbox;
}

